@@ -11,7 +11,7 @@ from abc import abstractmethod, abstractproperty
 from contextlib import ExitStack
 from ..._util import PygtailLogger
 from ..._util import StructuredLogMessage as _
-from .. import config, _statusbar
+from .. import config
 from ..error import Terminated, DownloadFailed
 from ..task_container import TaskContainer
 
@@ -61,6 +61,8 @@ class SubprocessBase(TaskContainer):
                     mem_bytes=memory_reservation,
                 )
             )
+            if self.runtime_values.get("gpu", False):
+                logger.warning("ignored runtime.gpu (not yet implemented)")
 
             # pull image if needed
             image = self._pull(logger, cleanup)
@@ -69,6 +71,7 @@ class SubprocessBase(TaskContainer):
             cli_log_filename = os.path.join(self.host_dir, f"{self.cli_name}.log.txt")
             cli_log = cleanup.enter_context(open(cli_log_filename, "wb"))
             cli_logger = logger.getChild(self.cli_name)
+            poll_stderr = cleanup.enter_context(self.poll_stderr_context(logger))
             poll_cli_log = cleanup.enter_context(
                 PygtailLogger(
                     logger,
@@ -77,13 +80,6 @@ class SubprocessBase(TaskContainer):
                     level=logging.INFO,
                 )
             )
-            poll_stderr = cleanup.enter_context(
-                PygtailLogger(
-                    logger,
-                    self.host_stderr_txt(),
-                    callback=self.stderr_callback,
-                )
-            )
 
             # prepare command
             with open(os.path.join(self.host_dir, "command"), "w") as outfile:
@@ -93,7 +89,7 @@ class SubprocessBase(TaskContainer):
             invocation = self._run_invocation(logger, cleanup, image) + [
                 "/bin/bash",
                 "-c",
-                "bash ../command >> ../stdout.txt 2>> ../stderr.txt",
+                "bash -l ../command >> ../stdout.txt 2>> ../stderr.txt",
             ]
             proc = subprocess.Popen(
                 invocation, stdout=cli_log, stderr=subprocess.STDOUT, cwd=self.host_dir
@@ -101,12 +97,7 @@ class SubprocessBase(TaskContainer):
             logger.notice(  # pyre-ignore
                 _(f"{self.cli_name} run", pid=proc.pid, log=cli_log_filename)
             )
-            cleanup.enter_context(
-                _statusbar.task_running(
-                    self.runtime_values.get("cpu", 0),
-                    self.runtime_values.get("memory_reservation", 0),
-                )
-            )
+            cleanup.enter_context(self.task_running_context())
 
             # long-poll for completion
             exit_code = None
@@ -151,23 +142,14 @@ class SubprocessBase(TaskContainer):
         self._bind_input_files = False
 
     def prepare_mounts(self) -> List[Tuple[str, str, bool]]:
-        def touch_mount_point(host_path: str) -> None:
-            # touching each mount point ensures they'll be owned by invoking user:group
-            assert host_path.startswith(self.host_dir + "/")
-            if host_path.endswith("/"):
-                os.makedirs(host_path, exist_ok=True)
-            else:
-                os.makedirs(os.path.dirname(host_path), exist_ok=True)
-                with open(host_path, "x") as _:
-                    pass
 
         mounts = []
         # mount stdout, stderr, and working directory read/write
-        touch_mount_point(self.host_stdout_txt())
+        self.touch_mount_point(self.host_stdout_txt())
         mounts.append(
             (os.path.join(self.container_dir, "stdout.txt"), self.host_stdout_txt(), True)
         )
-        touch_mount_point(self.host_stderr_txt())
+        self.touch_mount_point(self.host_stderr_txt())
         mounts.append(
             (os.path.join(self.container_dir, "stderr.txt"), self.host_stderr_txt(), True)
         )
@@ -188,7 +170,7 @@ class SubprocessBase(TaskContainer):
                     self.host_dir, os.path.relpath(container_path.rstrip("/"), self.container_dir)
                 )
                 if not os.path.exists(host_mount_point):
-                    touch_mount_point(
+                    self.touch_mount_point(
                         host_mount_point + ("/" if container_path.endswith("/") else "")
                     )
                 mounts.append((container_path.rstrip("/"), host_path.rstrip("/"), False))
@@ -213,6 +195,9 @@ class SubprocessBase(TaskContainer):
                     logger.info(_(f"{self.cli_name} image already pulled", image=image))
                     return image
 
+            if not invocation:
+                # No action required, image could be cached externally.
+                return image
             logger.info(_(f"begin {self.cli_name} pull", command=" ".join(invocation)))
             try:
                 subprocess.run(
