@@ -53,6 +53,7 @@ from .._util import (
     LoggingFileHandler,
     compose_coroutines,
     pathsize,
+    path_really_within,
 )
 from .._util import StructuredLogMessage as _
 from . import config, _statusbar
@@ -133,7 +134,11 @@ class StateMachine:
     finished: Set[str]
     running: Set[str]
     waiting: Set[str]
-    filename_whitelist: Set[str]
+
+    # File/Directory paths that were either expressly supplied as workflow inputs, or were
+    # generated by this workflow execution. By default, these are the only local paths the workflow
+    # is allowed to access. (Unless [file_io] allow_any_input = true)
+    fspath_allowlist: Set[str]
     # TODO: factor out WorkflowState interface?
 
     def __init__(
@@ -155,7 +160,7 @@ class StateMachine:
         self.finished = set()
         self.running = set()
         self.waiting = set()
-        self.filename_whitelist = _fspaths(inputs)
+        self.fspath_allowlist = _fspaths(inputs)
 
         from .. import values_to_json
 
@@ -313,7 +318,7 @@ class StateMachine:
         call_node = self.jobs[job_id].node
         assert isinstance(call_node, Tree.Call)
         self.job_outputs[job_id] = outputs.wrap_namespace(call_node.name)
-        self.filename_whitelist |= _fspaths(outputs)
+        self.fspath_allowlist |= _fspaths(outputs)
         self.finished.add(job_id)
         self.running.remove(job_id)
 
@@ -370,7 +375,9 @@ class StateMachine:
             return Env.Bindings(Env.Binding(job.node.name, v))
 
         if isinstance(job.node, WorkflowOutputs):
-            return env
+            return Value.rewrite_env_paths(
+                env, lambda v: _check_path_allowed(cfg, self.fspath_allowlist, "workflow output", v)
+            )
 
         if isinstance(job.node, Tree.Call):
             # evaluate input expressions
@@ -401,17 +408,12 @@ class StateMachine:
                     ),
                 )
             )
-            # check input files against whitelist
-            disallowed_filenames = _fspaths(call_inputs) - self.filename_whitelist
-            disallowed_filenames = set(
-                fn
-                for fn in disallowed_filenames
-                if not downloadable(cfg, fn, directory=fn.endswith("/"))
+            call_inputs = Value.rewrite_env_paths(
+                call_inputs,
+                lambda v: _check_path_allowed(
+                    cfg, self.fspath_allowlist, f"call {job.node.name} input", v
+                ),
             )
-            if disallowed_filenames:
-                raise Error.InputError(
-                    f"call {job.node.name} inputs use unknown file: {next(iter(disallowed_filenames))}"
-                )
             # issue CallInstructions
             self.logger.notice(_("ready", job=job.id, callee=job.node.callee.name))  # pyre-fixme
             inplog = self.values_to_json(call_inputs)
@@ -594,7 +596,7 @@ def _gather(
 
 
 class _StdLib(StdLib.Base):
-    "checks against & updates the filename whitelist for the read_* and write_* functions"
+    "checks against & updates the file/directory allowlist for the read_* and write_* functions"
     cfg: config.Loader
     state: StateMachine
     cache: CallCache
@@ -612,15 +614,89 @@ class _StdLib(StdLib.Base):
             cached = self.cache.get_download(filename)
             if cached:
                 return cached
-        if filename in self.state.filename_whitelist:
-            return filename
-        raise Error.InputError("attempted read from unknown or inaccessible file " + filename)
+        return _check_path_allowed(
+            self.cfg, self.state.fspath_allowlist, "read_*() argument", Value.File(filename)
+        )
 
     def _virtualize_filename(self, filename: str) -> str:
-        self.state.filename_whitelist.add(filename)
+        self.state.fspath_allowlist.add(filename)
         return filename
 
 
+def _check_path_allowed(
+    cfg: config.Loader, allowlist: Set[str], desc: str, v: Union[Value.File, Value.Directory]
+) -> str:
+    isdir = isinstance(v, Value.Directory)
+    fspath = v.value.rstrip("/") + ("/" if isdir else "")
+    if fspath in allowlist or downloadable(cfg, fspath, directory=isdir):
+        return v.value
+    if not cfg.get_bool("file_io", "allow_any_input"):
+        raise Error.InputError(
+            desc + " uses file/directory not expressly supplied with workflow inputs"
+            " (to allow, set [file_io] allow_any_input = true): " + fspath
+        )
+    # allow_any_input: checks that normally happen in CLI.validate_input_path
+    if not (os.path.isdir(fspath) if isdir else os.path.isfile(fspath)):
+        raise Error.InputError(f"{desc} uses nonexistent file/directory: {fspath}")
+    fspath = os.path.abspath(fspath).rstrip("/")
+    if not path_really_within(fspath, cfg["file_io"]["root"]):
+        raise Error.InputError(
+            f"{desc} {v.value} must reside within [file_io] root " + cfg["file_io"]["root"]
+        )
+    return fspath
+
+
+class _ThreadPools:
+    # Singleton managing the thread pools for concurrent task and subworkflow execution
+    #
+    # All tasks run on one thread pool.
+    #
+    # Each subworkflow call runs on a thread pool reserved for its nested call depth level.
+    # (If we kept just one subworkflow thread pool, then nested calls could deadlock when no thread
+    # is available to run a subworkflow whilst the caller blocks its own thread.)
+    _lock: threading.Lock
+    _cleanup: ExitStack
+    _task_pool: futures.ThreadPoolExecutor
+    _subworkflow_pools: List[futures.ThreadPoolExecutor]
+    _subworkflow_concurrency: int
+
+    def __init__(self, cfg: config.Loader, cleanup: ExitStack) -> None:
+        self._lock = threading.Lock()
+        self._cleanup = cleanup.enter_context(ExitStack())
+
+        task_concurrency = (
+            cfg["scheduler"].get_int("task_concurrency")
+            or (
+                cfg.get_int("scheduler", "call_concurrency")  # pre-v1.5.4 legacy
+                if cfg.has_option("scheduler", "call_concurrency")
+                else 0
+            )
+            or multiprocessing.cpu_count()
+        )
+
+        self._task_pool = futures.ThreadPoolExecutor(max_workers=task_concurrency)
+        self._cleanup.callback(futures.ThreadPoolExecutor.shutdown, self._task_pool)
+
+        self._subworkflow_concurrency = cfg.get_int("scheduler", "subworkflow_concurrency") or max(
+            task_concurrency, multiprocessing.cpu_count()
+        )
+        self._subworkflow_pools = []
+
+    def submit_task(self, *args, **kwargs):  # pyre-ignore
+        with self._lock:
+            return self._task_pool.submit(*args, **kwargs)
+
+    def submit_subworkflow(self, call_depth: int, *args, **kwargs):  # pyre-ignore
+        with self._lock:
+            if call_depth >= len(self._subworkflow_pools):
+                # First time at this call depth -- initialize a thread pool for it
+                assert call_depth == len(self._subworkflow_pools)
+                pool = futures.ThreadPoolExecutor(self._subworkflow_concurrency)
+                self._cleanup.callback(futures.ThreadPoolExecutor.shutdown, pool)
+                self._subworkflow_pools.append(pool)
+            return self._subworkflow_pools[call_depth].submit(*args, **kwargs)
+
+
 def run_local_workflow(
     cfg: config.Loader,
     workflow: Tree.Workflow,
@@ -628,7 +704,7 @@ def run_local_workflow(
     run_id: Optional[str] = None,
     run_dir: Optional[str] = None,
     logger_prefix: Optional[List[str]] = None,
-    _thread_pools: Optional[Tuple[futures.ThreadPoolExecutor, futures.ThreadPoolExecutor]] = None,
+    _thread_pools: Optional[_ThreadPools] = None,
     _cache: Optional[CallCache] = None,
     _test_pickle: bool = False,
     _run_id_stack: Optional[List[str]] = None,
@@ -728,19 +804,7 @@ def run_local_workflow(
                 version = "UNKNOWN"
             logger.notice(_("miniwdl", version=version, uname=" ".join(os.uname())))  # pyre-fixme
 
-            # Provision separate thread pools for tasks and sub-workflows. With just one pool, it'd
-            # be possible for all threads to be taken up by sub-workflows, deadlocking with no
-            # threads available to actually run their tasks.
-            # There's still a minor risk of deadlock if sub-workflow nesting is deeper than the
-            # subworkflow thread pool size.
-            max_workers = (
-                cfg["scheduler"].get_int("call_concurrency") or multiprocessing.cpu_count()
-            )
-            task_pool = futures.ThreadPoolExecutor(max_workers=max_workers)
-            cleanup.callback(futures.ThreadPoolExecutor.shutdown, task_pool)
-            subwf_pool = futures.ThreadPoolExecutor(max_workers=max_workers)
-            cleanup.callback(futures.ThreadPoolExecutor.shutdown, subwf_pool)
-            thread_pools = (task_pool, subwf_pool)
+            thread_pools = _ThreadPools(cfg, cleanup)
         else:
             assert _run_id_stack and _cache
             thread_pools = _thread_pools
@@ -782,7 +846,7 @@ def _workflow_main_loop(
     run_dir: str,
     logger: logging.Logger,
     logger_id: List[str],
-    thread_pools: Tuple[futures.ThreadPoolExecutor, futures.ThreadPoolExecutor],
+    thread_pools: _ThreadPools,
     cache: CallCache,
     terminating: Callable[[], bool],
     _test_pickle: bool,
@@ -812,7 +876,7 @@ def _workflow_main_loop(
                 logger_id,
                 run_dir,
                 _add_downloadable_defaults(cfg, workflow.available_inputs, inputs),
-                thread_pools[0],
+                thread_pools,
                 cache,
             )
 
@@ -843,10 +907,14 @@ def _workflow_main_loop(
                     # submit to appropriate thread pool
                     if isinstance(next_call.callee, Tree.Task):
                         _statusbar.task_backlogged()
-                        future = thread_pools[0].submit(run_local_task, *sub_args, **sub_kwargs)
+                        future = thread_pools.submit_task(run_local_task, *sub_args, **sub_kwargs)
                     elif isinstance(next_call.callee, Tree.Workflow):
-                        future = thread_pools[1].submit(
-                            run_local_workflow, *sub_args, **sub_kwargs, _thread_pools=thread_pools
+                        future = thread_pools.submit_subworkflow(
+                            len(run_id_stack) - 1,
+                            run_local_workflow,
+                            *sub_args,
+                            **sub_kwargs,
+                            _thread_pools=thread_pools,
                         )
                     else:
                         assert False
@@ -925,7 +993,7 @@ def _download_input_files(
     logger_prefix: List[str],
     run_dir: str,
     inputs: Env.Bindings[Value.Base],
-    thread_pool: futures.ThreadPoolExecutor,
+    thread_pools: _ThreadPools,
     cache: CallCache,
 ) -> None:
     """
@@ -973,7 +1041,7 @@ def _download_input_files(
             logger.info(
                 _(f"schedule input {'directory' if directory else 'file'} download", uri=uri)
             )
-            future = thread_pool.submit(
+            future = thread_pools.submit_task(
                 download,
                 cfg,
                 logger,
