@@ -619,7 +619,7 @@ class TestDownload(RunnerTestCase):
                 self.assertTrue("'cached': 2" in msg)
 
     @log_capture()
-    def test_directory(self, capture):
+    def test_aws_directory(self, capture):
         wdl6 = R"""
         version development
         workflow count_dir {
@@ -681,6 +681,68 @@ class TestDownload(RunnerTestCase):
         assert next((msg for msg in new_logs if "found in download cache" in msg), False)
         logs += new_logs
 
+    @log_capture()
+    def test_gsutil_directory(self, capture):
+        wdl6 = R"""
+        version development
+        workflow count_dir {
+            input {
+                Directory dir
+            }
+            call directory_files {
+                input:
+                    dir = dir
+            }
+            output {
+                Int file_count = length(directory_files.files)
+            }
+        }
+        task directory_files {
+            input {
+                Directory dir
+            }
+            command {
+                find "~{dir}" -type f > files.txt
+                >&2 cat files.txt
+            }
+            output {
+                Array[File] files = read_lines("files.txt")
+            }
+        }
+        """
+
+        # uncached
+        inp = {"dir": "gs://genomics-public-data/ftp-trace.ncbi.nih.gov/1000genomes/ftp/phase3/integrated_sv_map/supporting/breakpoints/"}
+        outp = self._run(wdl6, inp, task="directory_files")
+        self.assertEqual(len(outp["files"]), 4)
+
+        outp = self._run(wdl6, inp)
+        self.assertEqual(outp["file_count"], 4)
+        logs = [str(record.msg) for record in capture.records]
+
+        # cached
+        cfg = WDL.runtime.config.Loader(logging.getLogger(self.id()))
+        cfg.override({
+            "download_cache": {
+                "put": True,
+                "get": True,
+                "dir": os.path.join(self._dir, "cache6")
+            },
+            "logging": { "json": True }
+        })
+        self._run(wdl6, inp, cfg=cfg)
+        new_logs = [str(record.msg) for record in capture.records][len(logs):]
+        assert "'downloaded': 1" in next(msg for msg in new_logs if "processed input URIs" in msg), str(logs)
+        logs += new_logs
+        self._run(wdl6, inp, cfg=cfg)
+        new_logs = [str(record.msg) for record in capture.records][len(logs):]
+        assert next((msg for msg in new_logs if "found in download cache" in msg), False)
+        logs += new_logs
+        outp = self._run(wdl6, inp, task="directory_files", cfg=cfg)
+        self.assertEqual(len(outp["files"]), 4)
+        new_logs = [str(record.msg) for record in capture.records][len(logs):]
+        assert next((msg for msg in new_logs if "found in download cache" in msg), False)
+        logs += new_logs
 
 class RuntimeOverride(RunnerTestCase):
     def test_runtime_override(self):
@@ -1059,3 +1121,107 @@ passthru_test_success
 set123
 """,
         )
+
+class TestDockerNetwork(RunnerTestCase):
+    @classmethod
+    def setUpClass(self):
+        super().setUpClass()
+        self.subnet = "192.168.99.0/24"
+        self.network_name = "miniwdl_test7_net"
+        self.client = docker.from_env(version="auto")
+        try:
+            self.network = self.client.networks.get(self.network_name)
+        except docker.errors.NotFound:
+            ipam_pool = docker.types.IPAMPool(
+                subnet=self.subnet,
+            )
+            ipam_config = docker.types.IPAMConfig(pool_configs=[ipam_pool])
+            # We need a swarm-compatible docker network.
+            self.network = self.client.networks.create(
+                self.network_name, "overlay", ipam=ipam_config
+            )
+
+
+    @classmethod
+    def tearDownClass(self):
+        super().tearDownClass()
+        self.network.remove()
+        self.client.close()
+
+
+    def test_network_default(self):
+        wdl = """
+        version development
+        task t {
+            command <<<
+                hostname -I
+            >>>
+            output {
+                String out = read_string(stdout())
+            }
+            runtime {
+                docker: "ubuntu:20.04"
+            }
+        }
+        """
+        cfg = WDL.runtime.config.Loader(logging.getLogger(self.id()), [])
+        cfg.override(
+            {"task_runtime": {"defaults": {"docker_network": self.network_name}},
+             "docker_swarm": {"allow_networks": [self.network_name]}}
+        )
+        out = self._run(wdl, {}, cfg=cfg)
+        self.assertEqual(out["out"][:11], "192.168.99.")
+
+
+    @log_capture()
+    def test_network_explicit(self, capture):
+        wdl = f"""
+        version development
+        task t {{
+            command <<<
+                hostname -I
+            >>>
+            output {{
+                String out = read_string(stdout())
+            }}
+            runtime {{
+                docker_network: "{self.network_name}"
+                docker: "ubuntu:20.04"
+            }}
+        }}
+        """
+        cfg = WDL.runtime.config.Loader(logging.getLogger(self.id()), [])
+        cfg.override(
+            {"docker_swarm": {"allow_networks": [self.network_name]}}
+        )
+        out = self._run(wdl, {}, cfg=cfg)
+        self.assertEqual(out["out"][:11], "192.168.99.")
+
+        # make sure allowlist is effective
+        out = self._run(wdl, {})
+        self.assertNotEqual(out["out"][:11], "192.168.99.")
+        logs = "\n".join(str(record.msg) for record in capture.records)
+        assert "runtime.docker_network ignored" in logs
+
+
+    def test_network_host(self):
+        wdl = """
+        version development
+        task t {
+            command <<<
+                hostname -I
+            >>>
+            output {
+                String out = read_string(stdout())
+            }
+            runtime {
+                docker: "ubuntu:20.04"
+                docker_network: "host"
+            }
+        }
+        """
+        cfg = WDL.runtime.config.Loader(logging.getLogger(self.id()), [])
+        cfg.override(
+            {"docker_swarm": {"allow_networks": ["host"]}}
+        )
+        self._run(wdl, {}, cfg=cfg)
